@@ -16,7 +16,6 @@
 #ifdef KWIN_BUILD_ACTIVITIES
 #include "activities.h"
 #endif
-#include "decorationitem.h"
 #include "deleted.h"
 #include "x11client.h"
 #include "cursor.h"
@@ -32,16 +31,15 @@
 #include "scripting/scriptedeffect.h"
 #include "screens.h"
 #include "screenlockerwatcher.h"
-#include "surfaceitem.h"
-#include "thumbnailitem.h"
 #include "virtualdesktops.h"
 #include "window_property_notify_x11_filter.h"
-#include "windowitem.h"
 #include "workspace.h"
 #include "kwinglutils.h"
 #include "kwineffectquickview.h"
 
 #include <QDebug>
+#include <QMouseEvent>
+#include <QWheelEvent>
 
 #include <Plasma/Theme>
 
@@ -113,7 +111,6 @@ EffectsHandlerImpl::EffectsHandlerImpl(Compositor *compositor, Scene *scene)
     : EffectsHandler(scene->compositingType())
     , keyboard_grab_effect(nullptr)
     , fullscreen_effect(nullptr)
-    , next_window_quad_type(EFFECT_QUAD_TYPE_START)
     , m_compositor(compositor)
     , m_scene(scene)
     , m_desktopRendering(false)
@@ -133,8 +130,6 @@ EffectsHandlerImpl::EffectsHandlerImpl(Compositor *compositor, Scene *scene)
     new EffectsAdaptor(this);
     QDBusConnection dbus = QDBusConnection::sessionBus();
     dbus.registerObject(QStringLiteral("/Effects"), this);
-    // init is important, otherwise causes crashes when quads are build before the first painting pass start
-    m_currentBuildQuadsIterator = m_activeEffects.constEnd();
 
     Workspace *ws = Workspace::self();
     VirtualDesktopManager *vds = VirtualDesktopManager::self();
@@ -144,9 +139,9 @@ EffectsHandlerImpl::EffectsHandlerImpl(Compositor *compositor, Scene *scene)
         [this](int old, AbstractClient *c) {
             const int newDesktop = VirtualDesktopManager::self()->current();
             if (old != 0 && newDesktop != old) {
-                emit desktopChanged(old, newDesktop, c ? c->effectWindow() : nullptr);
+                Q_EMIT desktopChanged(old, newDesktop, c ? c->effectWindow() : nullptr);
                 // TODO: remove in 4.10
-                emit desktopChanged(old, newDesktop);
+                Q_EMIT desktopChanged(old, newDesktop);
             }
         }
     );
@@ -155,7 +150,7 @@ EffectsHandlerImpl::EffectsHandlerImpl(Compositor *compositor, Scene *scene)
             if (!c->effectWindow()) {
                 return;
             }
-            emit desktopPresenceChanged(c->effectWindow(), old, c->desktop());
+            Q_EMIT desktopPresenceChanged(c->effectWindow(), old, c->desktop());
         }
     );
     connect(ws, &Workspace::clientAdded, this,
@@ -175,17 +170,17 @@ EffectsHandlerImpl::EffectsHandlerImpl(Compositor *compositor, Scene *scene)
     connect(ws, &Workspace::internalClientAdded, this,
         [this](InternalClient *client) {
             setupClientConnections(client);
-            emit windowAdded(client->effectWindow());
+            Q_EMIT windowAdded(client->effectWindow());
         }
     );
     connect(ws, &Workspace::clientActivated, this,
         [this](KWin::AbstractClient *c) {
-            emit windowActivated(c ? c->effectWindow() : nullptr);
+            Q_EMIT windowActivated(c ? c->effectWindow() : nullptr);
         }
     );
     connect(ws, &Workspace::deletedRemoved, this,
         [this](KWin::Deleted *d) {
-            emit windowDeleted(d->effectWindow());
+            Q_EMIT windowDeleted(d->effectWindow());
             elevated_windows.removeAll(d->effectWindow());
         }
     );
@@ -232,7 +227,7 @@ EffectsHandlerImpl::EffectsHandlerImpl(Compositor *compositor, Scene *scene)
             } else {
                 m_x11WindowPropertyNotify.reset();
             }
-            emit xcbConnectionChanged();
+            Q_EMIT xcbConnectionChanged();
         }
     );
 
@@ -273,7 +268,7 @@ EffectsHandlerImpl::~EffectsHandlerImpl()
 
 void EffectsHandlerImpl::unloadAllEffects()
 {
-    for (const EffectPair &pair : loaded_effects) {
+    for (const EffectPair &pair : qAsConst(loaded_effects)) {
         destroyEffect(pair.second);
     }
 
@@ -290,17 +285,17 @@ void EffectsHandlerImpl::setupClientConnections(AbstractClient* c)
             this, &EffectsHandlerImpl::slotClientMaximized);
     connect(c, &AbstractClient::clientStartUserMovedResized, this,
         [this](AbstractClient *c) {
-            emit windowStartUserMovedResized(c->effectWindow());
+            Q_EMIT windowStartUserMovedResized(c->effectWindow());
         }
     );
     connect(c, &AbstractClient::clientStepUserMovedResized, this,
         [this](AbstractClient *c, const QRect &geometry) {
-            emit windowStepUserMovedResized(c->effectWindow(), geometry);
+            Q_EMIT windowStepUserMovedResized(c->effectWindow(), geometry);
         }
     );
     connect(c, &AbstractClient::clientFinishUserMovedResized, this,
         [this](AbstractClient *c) {
-            emit windowFinishUserMovedResized(c->effectWindow());
+            Q_EMIT windowFinishUserMovedResized(c->effectWindow());
         }
     );
     connect(c, &AbstractClient::opacityChanged, this, &EffectsHandlerImpl::slotOpacityChanged);
@@ -308,7 +303,7 @@ void EffectsHandlerImpl::setupClientConnections(AbstractClient* c)
         [this](AbstractClient *c, bool animate) {
             // TODO: notify effects even if it should not animate?
             if (animate) {
-                emit windowMinimized(c->effectWindow());
+                Q_EMIT windowMinimized(c->effectWindow());
             }
         }
     );
@@ -316,7 +311,7 @@ void EffectsHandlerImpl::setupClientConnections(AbstractClient* c)
         [this](AbstractClient* c, bool animate) {
             // TODO: notify effects even if it should not animate?
             if (animate) {
-                emit windowUnminimized(c->effectWindow());
+                Q_EMIT windowUnminimized(c->effectWindow());
             }
         }
     );
@@ -326,38 +321,38 @@ void EffectsHandlerImpl::setupClientConnections(AbstractClient* c)
     connect(c, &AbstractClient::damaged,              this, &EffectsHandlerImpl::slotWindowDamaged);
     connect(c, &AbstractClient::unresponsiveChanged, this,
         [this, c](bool unresponsive) {
-            emit windowUnresponsiveChanged(c->effectWindow(), unresponsive);
+            Q_EMIT windowUnresponsiveChanged(c->effectWindow(), unresponsive);
         }
     );
     connect(c, &AbstractClient::windowShown, this,
         [this](Toplevel *c) {
-            emit windowShown(c->effectWindow());
+            Q_EMIT windowShown(c->effectWindow());
         }
     );
     connect(c, &AbstractClient::windowHidden, this,
         [this](Toplevel *c) {
-            emit windowHidden(c->effectWindow());
+            Q_EMIT windowHidden(c->effectWindow());
         }
     );
     connect(c, &AbstractClient::keepAboveChanged, this,
         [this, c](bool above) {
             Q_UNUSED(above)
-            emit windowKeepAboveChanged(c->effectWindow());
+            Q_EMIT windowKeepAboveChanged(c->effectWindow());
         }
     );
     connect(c, &AbstractClient::keepBelowChanged, this,
         [this, c](bool below) {
             Q_UNUSED(below)
-            emit windowKeepBelowChanged(c->effectWindow());
+            Q_EMIT windowKeepBelowChanged(c->effectWindow());
         }
     );
     connect(c, &AbstractClient::fullScreenChanged, this,
         [this, c]() {
-            emit windowFullScreenChanged(c->effectWindow());
+            Q_EMIT windowFullScreenChanged(c->effectWindow());
         }
     );
     connect(c, &AbstractClient::visibleGeometryChanged, this, [this, c]() {
-        emit windowExpandedGeometryChanged(c->effectWindow());
+        Q_EMIT windowExpandedGeometryChanged(c->effectWindow());
     });
 }
 
@@ -369,7 +364,7 @@ void EffectsHandlerImpl::setupUnmanagedConnections(Unmanaged* u)
     connect(u, &Unmanaged::frameGeometryChanged, this, &EffectsHandlerImpl::slotFrameGeometryChanged);
     connect(u, &Unmanaged::damaged,              this, &EffectsHandlerImpl::slotWindowDamaged);
     connect(u, &Unmanaged::visibleGeometryChanged, this, [this, u]() {
-        emit windowExpandedGeometryChanged(u->effectWindow());
+        Q_EMIT windowExpandedGeometryChanged(u->effectWindow());
     });
 }
 
@@ -477,21 +472,6 @@ void EffectsHandlerImpl::drawWindow(EffectWindow* w, int mask, const QRegion &re
         m_scene->finalDrawWindow(static_cast<EffectWindowImpl*>(w), mask, region, data);
 }
 
-void EffectsHandlerImpl::buildQuads(EffectWindow* w, WindowQuadList& quadList)
-{
-    static bool initIterator = true;
-    if (initIterator) {
-        m_currentBuildQuadsIterator = m_activeEffects.constBegin();
-        initIterator = false;
-    }
-    if (m_currentBuildQuadsIterator != m_activeEffects.constEnd()) {
-        (*m_currentBuildQuadsIterator++)->buildQuads(w, quadList);
-        --m_currentBuildQuadsIterator;
-    }
-    if (m_currentBuildQuadsIterator == m_activeEffects.constBegin())
-        initIterator = true;
-}
-
 bool EffectsHandlerImpl::hasDecorationShadows() const
 {
     return false;
@@ -544,7 +524,7 @@ void EffectsHandlerImpl::slotClientMaximized(KWin::AbstractClient *c, MaximizeMo
         break;
     }
     if (EffectWindowImpl *w = c->effectWindow()) {
-        emit windowMaximizedStateChanged(w, horizontal, vertical);
+        Q_EMIT windowMaximizedStateChanged(w, horizontal, vertical);
     }
 }
 
@@ -553,7 +533,7 @@ void EffectsHandlerImpl::slotOpacityChanged(Toplevel *t, qreal oldOpacity)
     if (t->opacity() == oldOpacity || !t->effectWindow()) {
         return;
     }
-    emit windowOpacityChanged(t->effectWindow(), oldOpacity, (qreal)t->opacity());
+    Q_EMIT windowOpacityChanged(t->effectWindow(), oldOpacity, (qreal)t->opacity());
 }
 
 void EffectsHandlerImpl::slotClientShown(KWin::Toplevel *t)
@@ -562,7 +542,7 @@ void EffectsHandlerImpl::slotClientShown(KWin::Toplevel *t)
     AbstractClient *c = static_cast<AbstractClient *>(t);
     disconnect(c, &Toplevel::windowShown, this, &EffectsHandlerImpl::slotClientShown);
     setupClientConnections(c);
-    emit windowAdded(c->effectWindow());
+    Q_EMIT windowAdded(c->effectWindow());
 }
 
 void EffectsHandlerImpl::slotUnmanagedShown(KWin::Toplevel *t)
@@ -570,35 +550,35 @@ void EffectsHandlerImpl::slotUnmanagedShown(KWin::Toplevel *t)
     Q_ASSERT(qobject_cast<Unmanaged *>(t));
     Unmanaged *u = static_cast<Unmanaged*>(t);
     setupUnmanagedConnections(u);
-    emit windowAdded(u->effectWindow());
+    Q_EMIT windowAdded(u->effectWindow());
 }
 
 void EffectsHandlerImpl::slotWindowClosed(KWin::Toplevel *c, KWin::Deleted *d)
 {
     c->disconnect(this);
     if (d) {
-        emit windowClosed(c->effectWindow());
+        Q_EMIT windowClosed(c->effectWindow());
     }
 }
 
 void EffectsHandlerImpl::slotClientModalityChanged()
 {
-    emit windowModalityChanged(static_cast<X11Client *>(sender())->effectWindow());
+    Q_EMIT windowModalityChanged(static_cast<X11Client *>(sender())->effectWindow());
 }
 
 void EffectsHandlerImpl::slotCurrentTabAboutToChange(EffectWindow *from, EffectWindow *to)
 {
-    emit currentTabAboutToChange(from, to);
+    Q_EMIT currentTabAboutToChange(from, to);
 }
 
 void EffectsHandlerImpl::slotTabAdded(EffectWindow* w, EffectWindow* to)
 {
-    emit tabAdded(w, to);
+    Q_EMIT tabAdded(w, to);
 }
 
 void EffectsHandlerImpl::slotTabRemoved(EffectWindow *w, EffectWindow* leaderOfFormerGroup)
 {
-    emit tabRemoved(w, leaderOfFormerGroup);
+    Q_EMIT tabRemoved(w, leaderOfFormerGroup);
 }
 
 void EffectsHandlerImpl::slotWindowDamaged(Toplevel* t, const QRegion& r)
@@ -607,7 +587,7 @@ void EffectsHandlerImpl::slotWindowDamaged(Toplevel* t, const QRegion& r)
         // can happen during tear down of window
         return;
     }
-    emit windowDamaged(t->effectWindow(), r);
+    Q_EMIT windowDamaged(t->effectWindow(), r);
 }
 
 void EffectsHandlerImpl::slotGeometryShapeChanged(Toplevel* t, const QRect& old)
@@ -616,14 +596,14 @@ void EffectsHandlerImpl::slotGeometryShapeChanged(Toplevel* t, const QRect& old)
     // in some functions that may still call this
     if (t == nullptr || t->effectWindow() == nullptr)
         return;
-    emit windowGeometryShapeChanged(t->effectWindow(), old);
+    Q_EMIT windowGeometryShapeChanged(t->effectWindow(), old);
 }
 
 void EffectsHandlerImpl::slotFrameGeometryChanged(Toplevel *toplevel, const QRect &oldGeometry)
 {
     // effectWindow() might be nullptr during tear down of the client.
     if (toplevel->effectWindow()) {
-        emit windowFrameGeometryChanged(toplevel->effectWindow(), oldGeometry);
+        Q_EMIT windowFrameGeometryChanged(toplevel->effectWindow(), oldGeometry);
     }
 }
 
@@ -634,9 +614,9 @@ void EffectsHandlerImpl::setActiveFullScreenEffect(Effect* e)
     }
     const bool activeChanged = (e == nullptr || fullscreen_effect == nullptr);
     fullscreen_effect = e;
-    emit activeFullScreenEffectChanged();
+    Q_EMIT activeFullScreenEffectChanged();
     if (activeChanged) {
-        emit hasActiveFullScreenEffectChanged();
+        Q_EMIT hasActiveFullScreenEffectChanged();
     }
 }
 
@@ -770,6 +750,11 @@ void EffectsHandlerImpl::registerAxisShortcut(Qt::KeyboardModifiers modifiers, P
     input()->registerAxisShortcut(modifiers, axis, action);
 }
 
+void EffectsHandlerImpl::registerRealtimeTouchpadSwipeShortcut(SwipeDirection dir, QAction* onUp, std::function<void(qreal)> progressCallback)
+{
+    input()->registerRealtimeTouchpadSwipeShortcut(dir, onUp, progressCallback);
+}
+
 void EffectsHandlerImpl::registerTouchpadSwipeShortcut(SwipeDirection direction, QAction *action)
 {
     input()->registerTouchpadSwipeShortcut(direction, action);
@@ -801,12 +786,6 @@ bool EffectsHandlerImpl::hasKeyboardGrab() const
     return keyboard_grab_effect != nullptr;
 }
 
-void EffectsHandlerImpl::desktopResized(const QSize &size)
-{
-    m_scene->screenGeometryChanged(size);
-    emit screenGeometryChanged(size);
-}
-
 void EffectsHandlerImpl::registerPropertyType(long atom, bool reg)
 {
     if (reg)
@@ -925,9 +904,14 @@ void EffectsHandlerImpl::windowToDesktops(EffectWindow *w, const QVector<uint> &
 
 void EffectsHandlerImpl::windowToScreen(EffectWindow* w, int screen)
 {
+    AbstractOutput *output = kwinApp()->platform()->findOutput(screen);
+    if (!output) {
+        return;
+    }
     auto cl = qobject_cast<AbstractClient *>(static_cast<EffectWindowImpl *>(w)->window());
-    if (cl && !cl->isDesktop() && !cl->isDock())
-        Workspace::self()->sendClientToScreen(cl, screen);
+    if (cl && !cl->isDesktop() && !cl->isDock()) {
+        Workspace::self()->sendClientToOutput(cl, output);
+    }
 }
 
 void EffectsHandlerImpl::setShowingDesktop(bool showing)
@@ -1036,7 +1020,8 @@ int EffectsHandlerImpl::desktopToLeft(int desktop, bool wrap) const
 
 QString EffectsHandlerImpl::desktopName(int desktop) const
 {
-    return VirtualDesktopManager::self()->name(desktop);
+    const VirtualDesktop *vd = VirtualDesktopManager::self()->desktopForX11Id(desktop);
+    return vd ? vd->name() : QString();
 }
 
 bool EffectsHandlerImpl::optionRollOverDesktops() const
@@ -1049,11 +1034,6 @@ double EffectsHandlerImpl::animationTimeFactor() const
     return options->animationTimeFactor();
 }
 
-WindowQuadType EffectsHandlerImpl::newWindowQuadType()
-{
-    return WindowQuadType(next_window_quad_type++);
-}
-
 EffectWindow* EffectsHandlerImpl::findWindow(WId id) const
 {
     if (X11Client *w = Workspace::self()->findClient(Predicate::WindowMatch, id))
@@ -1215,7 +1195,7 @@ void EffectsHandlerImpl::addRepaint(int x, int y, int w, int h)
 
 int EffectsHandlerImpl::activeScreen() const
 {
-    return Screens::self()->current();
+    return kwinApp()->platform()->enabledOutputs().indexOf(workspace()->activeOutput());
 }
 
 int EffectsHandlerImpl::numScreens() const
@@ -1236,11 +1216,7 @@ QRect EffectsHandlerImpl::clientArea(clientAreaOption opt, int screen, int deskt
 QRect EffectsHandlerImpl::clientArea(clientAreaOption opt, const EffectWindow* c) const
 {
     const Toplevel* t = static_cast< const EffectWindowImpl* >(c)->window();
-    if (const auto *cl = qobject_cast<const AbstractClient *>(t)) {
-        return Workspace::self()->clientArea(opt, cl);
-    } else {
-        return Workspace::self()->clientArea(opt, t->frameGeometry().center(), VirtualDesktopManager::self()->current());
-    }
+    return Workspace::self()->clientArea(opt, t);
 }
 
 QRect EffectsHandlerImpl::clientArea(clientAreaOption opt, const QPoint& p, int desktop) const
@@ -1268,7 +1244,7 @@ bool EffectsHandlerImpl::checkInputWindowEvent(QMouseEvent *e)
     if (m_grabbedMouseEffects.isEmpty()) {
         return false;
     }
-    foreach (Effect *effect, m_grabbedMouseEffects) {
+    Q_FOREACH (Effect *effect, m_grabbedMouseEffects) {
         effect->windowInputMouseEvent(e);
     }
     return true;
@@ -1279,7 +1255,7 @@ bool EffectsHandlerImpl::checkInputWindowEvent(QWheelEvent *e)
     if (m_grabbedMouseEffects.isEmpty()) {
         return false;
     }
-    foreach (Effect *effect, m_grabbedMouseEffects) {
+    Q_FOREACH (Effect *effect, m_grabbedMouseEffects) {
         effect->windowInputMouseEvent(e);
     }
     return true;
@@ -1347,11 +1323,6 @@ void EffectsHandlerImpl::unregisterTouchBorder(ElectricBorder border, QAction *a
     ScreenEdges::self()->unreserveTouch(border, action);
 }
 
-unsigned long EffectsHandlerImpl::xrenderBufferPicture()
-{
-    return m_scene->xrenderBufferPicture();
-}
-
 QPainter *EffectsHandlerImpl::scenePainter()
 {
     return m_scene->scenePainter();
@@ -1733,7 +1704,7 @@ void EffectsHandlerImpl::slotOutputEnabled(AbstractOutput *output)
 {
     EffectScreen *screen = new EffectScreenImpl(output, this);
     m_effectScreens.append(screen);
-    emit screenAdded(screen);
+    Q_EMIT screenAdded(screen);
 }
 
 void EffectsHandlerImpl::slotOutputDisabled(AbstractOutput *output)
@@ -1744,11 +1715,17 @@ void EffectsHandlerImpl::slotOutputDisabled(AbstractOutput *output)
     if (it != m_effectScreens.end()) {
         EffectScreen *screen = *it;
         m_effectScreens.erase(it);
-        emit screenRemoved(screen);
+        Q_EMIT screenRemoved(screen);
         delete screen;
     }
 }
 
+void EffectsHandlerImpl::renderScreen(EffectScreen *screen)
+{
+    auto output = static_cast<EffectScreenImpl *>(screen)->platformOutput();
+    scene()->paintScreen(output, Compositor::self()->windowsToRender());
+}
+
 //****************************************
 // EffectScreenImpl
 //****************************************
@@ -1757,6 +1734,12 @@ EffectScreenImpl::EffectScreenImpl(AbstractOutput *output, QObject *parent)
     : EffectScreen(parent)
     , m_platformOutput(output)
 {
+    connect(output, &AbstractOutput::aboutToChange, this, &EffectScreen::aboutToChange);
+    connect(output, &AbstractOutput::changed, this, &EffectScreen::changed);
+    connect(output, &AbstractOutput::wakeUp, this, &EffectScreen::wakeUp);
+    connect(output, &AbstractOutput::aboutToTurnOff, this, &EffectScreen::aboutToTurnOff);
+    connect(output, &AbstractOutput::scaleChanged, this, &EffectScreen::devicePixelRatioChanged);
+    connect(output, &AbstractOutput::geometryChanged, this, &EffectScreen::geometryChanged);
 }
 
 AbstractOutput *EffectScreenImpl::platformOutput() const
@@ -1779,6 +1762,11 @@ QRect EffectScreenImpl::geometry() const
     return m_platformOutput->geometry();
 }
 
+EffectScreen::Transform EffectScreenImpl::transform() const
+{
+    return EffectScreen::Transform(m_platformOutput->transform());
+}
+
 //****************************************
 // EffectWindowImpl
 //****************************************
@@ -1893,6 +1881,7 @@ TOPLEVEL_HELPER(int, screen, screen)
 TOPLEVEL_HELPER(QRect, geometry, frameGeometry)
 TOPLEVEL_HELPER(QRect, frameGeometry, frameGeometry)
 TOPLEVEL_HELPER(QRect, bufferGeometry, bufferGeometry)
+TOPLEVEL_HELPER(QRect, clientGeometry, clientGeometry)
 TOPLEVEL_HELPER(QRect, expandedGeometry, visibleGeometry)
 TOPLEVEL_HELPER(QRect, rect, rect)
 TOPLEVEL_HELPER(int, desktop, desktop)
@@ -1913,7 +1902,6 @@ TOPLEVEL_HELPER(bool, isOnScreenDisplay, isOnScreenDisplay)
 TOPLEVEL_HELPER(bool, isComboBox, isComboBox)
 TOPLEVEL_HELPER(bool, isDNDIcon, isDNDIcon)
 TOPLEVEL_HELPER(bool, isDeleted, isDeleted)
-TOPLEVEL_HELPER(bool, hasOwnShape, shape)
 TOPLEVEL_HELPER(QString, windowRole, windowRole)
 TOPLEVEL_HELPER(QStringList, activities, activities)
 TOPLEVEL_HELPER(bool, skipsCloseAnimation, skipsCloseAnimation)
@@ -1983,8 +1971,8 @@ NET::WindowType EffectWindowImpl::windowType() const
 
 CLIENT_HELPER(bool, isMovable, isMovable, false)
 CLIENT_HELPER(bool, isMovableAcrossScreens, isMovableAcrossScreens, false)
-CLIENT_HELPER(bool, isUserMove, isMove, false)
-CLIENT_HELPER(bool, isUserResize, isResize, false)
+CLIENT_HELPER(bool, isUserMove, isInteractiveMove, false)
+CLIENT_HELPER(bool, isUserResize, isInteractiveResize, false)
 CLIENT_HELPER(QRect, iconGeometry, iconGeometry, QRect())
 CLIENT_HELPER(bool, isSpecialWindow, isSpecialWindow, true)
 CLIENT_HELPER(bool, acceptsFocus, wantsInput, true) // We don't actually know...
@@ -2014,23 +2002,9 @@ void EffectWindowImpl::setSceneWindow(Scene::Window* w)
     sw = w;
 }
 
-QRegion EffectWindowImpl::shape() const
-{
-    if (isX11Client()) {
-        const WindowItem *windowItem = sceneWindow()->windowItem();
-        if (DecorationItem *item = windowItem->decorationItem()) {
-            return item->rect();
-        } else if (SurfaceItem *item = windowItem->surfaceItem()) {
-            return item->shape();
-        }
-    }
-    return toplevel->rect();
-}
-
 QRect EffectWindowImpl::decorationInnerRect() const
 {
-    auto client = qobject_cast<X11Client *>(toplevel);
-    return client ? client->transparentRect() : contentsRect();
+    return toplevel->rect() - toplevel->frameMargins();
 }
 
 QByteArray EffectWindowImpl::readProperty(long atom, long type, int format) const
@@ -2110,18 +2084,13 @@ EffectWindowList EffectWindowImpl::mainWindows() const
     return {};
 }
 
-WindowQuadList EffectWindowImpl::buildQuads(bool force) const
-{
-    return sceneWindow()->buildQuads(force);
-}
-
 void EffectWindowImpl::setData(int role, const QVariant &data)
 {
     if (!data.isNull())
         dataMap[ role ] = data;
     else
         dataMap.remove(role);
-    emit effects->windowDataChanged(this, role);
+    Q_EMIT effects->windowDataChanged(this, role);
 }
 
 QVariant EffectWindowImpl::data(int role) const
@@ -2147,47 +2116,6 @@ void EffectWindowImpl::elevate(bool elevate)
     effects->setElevatedWindow(this, elevate);
 }
 
-void EffectWindowImpl::registerThumbnail(AbstractThumbnailItem *item)
-{
-    if (WindowThumbnailItem *thumb = qobject_cast<WindowThumbnailItem*>(item)) {
-        insertThumbnail(thumb);
-        connect(thumb, &QObject::destroyed, this, &EffectWindowImpl::thumbnailDestroyed);
-        connect(thumb, &WindowThumbnailItem::wIdChanged, this, &EffectWindowImpl::thumbnailTargetChanged);
-    } else if (DesktopThumbnailItem *desktopThumb = qobject_cast<DesktopThumbnailItem*>(item)) {
-        m_desktopThumbnails.append(desktopThumb);
-        connect(desktopThumb, &QObject::destroyed, this, &EffectWindowImpl::desktopThumbnailDestroyed);
-    }
-}
-
-void EffectWindowImpl::thumbnailDestroyed(QObject *object)
-{
-    // we know it is a ThumbnailItem
-    m_thumbnails.remove(static_cast<WindowThumbnailItem*>(object));
-}
-
-void EffectWindowImpl::thumbnailTargetChanged()
-{
-    if (WindowThumbnailItem *item = qobject_cast<WindowThumbnailItem*>(sender())) {
-        insertThumbnail(item);
-    }
-}
-
-void EffectWindowImpl::insertThumbnail(WindowThumbnailItem *item)
-{
-    EffectWindow *w = effects->findWindow(item->wId());
-    if (w) {
-        m_thumbnails.insert(item, QPointer<EffectWindowImpl>(static_cast<EffectWindowImpl*>(w)));
-    } else {
-        m_thumbnails.insert(item, QPointer<EffectWindowImpl>());
-    }
-}
-
-void EffectWindowImpl::desktopThumbnailDestroyed(QObject *object)
-{
-    // we know it is a DesktopThumbnailItem
-    m_desktopThumbnails.removeAll(static_cast<DesktopThumbnailItem*>(object));
-}
-
 void EffectWindowImpl::minimize()
 {
     if (auto client = qobject_cast<AbstractClient *>(toplevel)) {
@@ -2352,7 +2280,7 @@ void EffectFrameImpl::setIcon(const QIcon& icon)
         m_sceneFrame->crossFadeIcon();
     }
     if (m_iconSize.isEmpty() && !m_icon.availableSizes().isEmpty()) { // Set a size if we don't already have one
-        setIconSize(m_icon.availableSizes().first());
+        setIconSize(m_icon.availableSizes().constFirst());
     }
     m_sceneFrame->freeIconFrame();
 }
