@@ -52,7 +52,7 @@ BlurEffect::BlurEffect()
     connect(effects, &EffectsHandler::windowAdded, this, &BlurEffect::slotWindowAdded);
     connect(effects, &EffectsHandler::windowDeleted, this, &BlurEffect::slotWindowDeleted);
     connect(effects, &EffectsHandler::propertyNotify, this, &BlurEffect::slotPropertyNotify);
-    connect(effects, &EffectsHandler::screenGeometryChanged, this, &BlurEffect::slotScreenGeometryChanged);
+    connect(effects, &EffectsHandler::virtualScreenGeometryChanged, this, &BlurEffect::slotScreenGeometryChanged);
     connect(effects, &EffectsHandler::xcbConnectionChanged, this,
         [this] {
             if (m_shader && m_shader->isValid() && m_renderTargetsValid) {
@@ -62,7 +62,7 @@ BlurEffect::BlurEffect()
     );
 
     // Fetch the blur regions for all windows
-    foreach (EffectWindow *window, effects->stackingOrder())
+    Q_FOREACH (EffectWindow *window, effects->stackingOrder())
         updateBlurRegion(window);
 }
 
@@ -77,7 +77,7 @@ void BlurEffect::slotScreenGeometryChanged()
     updateTexture();
 
     // Fetch the blur regions for all windows
-    foreach (EffectWindow *window, effects->stackingOrder())
+    Q_FOREACH (EffectWindow *window, effects->stackingOrder())
         updateBlurRegion(window);
     effects->doneOpenGLContextCurrent();
 }
@@ -407,21 +407,19 @@ QRegion BlurEffect::blurRegion(const EffectWindow *w) const
         const QRegion appRegion = qvariant_cast<QRegion>(value);
         if (!appRegion.isEmpty()) {
             if (w->decorationHasAlpha() && effects->decorationSupportsBlurBehind()) {
-                region = w->shape() & w->rect();
-                region -= w->decorationInnerRect();
+                region = QRegion(w->rect()) - w->decorationInnerRect();
             }
             region |= appRegion.translated(w->contentsRect().topLeft()) &
                       w->decorationInnerRect();
         } else {
             // An empty region means that the blur effect should be enabled
             // for the whole window.
-            region = w->shape() & w->rect();
+            region = w->rect();
         }
     } else if (w->decorationHasAlpha() && effects->decorationSupportsBlurBehind()) {
         // If the client hasn't specified a blur region, we'll only enable
         // the effect behind the decoration.
-        region = w->shape() & w->rect();
-        region -= w->decorationInnerRect();
+        region = QRegion(w->rect()) - w->decorationInnerRect();
     }
 
     return region;
@@ -589,7 +587,7 @@ void BlurEffect::drawWindow(EffectWindow *w, int mask, const QRegion &region, Wi
         const bool transientForIsDock = (modal ? modal->isDock() : false);
 
         if (!shape.isEmpty()) {
-            doBlur(shape, screen, data.opacity(), data.screenProjectionMatrix(), w->isDock() || transientForIsDock, w->geometry());
+            doBlur(shape, screen, data.opacity(), data.screenProjectionMatrix(), w->isDock() || transientForIsDock, w->frameGeometry());
         }
     }
 
@@ -625,16 +623,16 @@ void BlurEffect::generateNoiseTexture()
         uint8_t *noiseImageLine = (uint8_t *) noiseImage.scanLine(y);
 
         for (int x = 0; x < noiseImage.width(); x++) {
-            noiseImageLine[x] = qrand() % m_noiseStrength + (128 - m_noiseStrength / 2);
+            noiseImageLine[x] = qrand() % m_noiseStrength;
         }
     }
 
     // The noise texture looks distorted when not scaled with integer
     noiseImage = noiseImage.scaled(noiseImage.size() * m_scalingFactor);
 
-    m_noiseTexture = GLTexture(noiseImage);
-    m_noiseTexture.setFilter(GL_NEAREST);
-    m_noiseTexture.setWrapMode(GL_REPEAT);
+    m_noiseTexture.reset(new GLTexture(noiseImage));
+    m_noiseTexture->setFilter(GL_NEAREST);
+    m_noiseTexture->setWrapMode(GL_REPEAT);
 }
 
 void BlurEffect::doBlur(const QRegion& shape, const QRect& screen, const float opacity, const QMatrix4x4 &screenProjection, bool isDock, QRect windowRect)
@@ -675,7 +673,10 @@ void BlurEffect::doBlur(const QRegion& shape, const QRect& screen, const float o
             glEnable(GL_FRAMEBUFFER_SRGB);
         }
 
-        copyScreenSampleTexture(vbo, blurRectCount, shape.translated(xTranslate, yTranslate), screenProjection);
+        const QRect screenRect = effects->virtualScreenGeometry();
+        QMatrix4x4 mvp;
+        mvp.ortho(0, screenRect.width(), screenRect.height(), 0, 0, 65535);
+        copyScreenSampleTexture(vbo, blurRectCount, shape.translated(xTranslate, yTranslate), mvp);
     } else {
         m_renderTargets.first()->blitFromFramebuffer(sourceRect, destRect);
 
@@ -714,34 +715,58 @@ void BlurEffect::doBlur(const QRegion& shape, const QRect& screen, const float o
         glDisable(GL_BLEND);
     }
 
+    if (m_noiseStrength > 0) {
+        // Apply an additive noise onto the blurred image.
+        // The noise is useful to mask banding artifacts, which often happens due to the smooth color transitions in the
+        // blurred image.
+        // The noise is applied in perceptual space (i.e. after glDisable(GL_FRAMEBUFFER_SRGB)). This practice is also
+        // seen in other application of noise synthesis (films, image codecs), and makes the noise less visible overall
+        // (reduces graininess).
+        glEnable(GL_BLEND);
+        if (opacity < 1.0) {
+            // We need to modulate the opacity of the noise as well; otherwise a thin layer would appear when applying
+            // effects like fade out.
+            // glBlendColor should have been set above.
+            glBlendFunc(GL_CONSTANT_ALPHA, GL_ONE);
+        } else {
+            // Add the shader's output directly to the pixels in framebuffer.
+            glBlendFunc(GL_ONE, GL_ONE);
+        }
+        applyNoise(vbo, blurRectCount * (m_downSampleIterations + 1), shape.rectCount() * 6, screenProjection, windowRect.topLeft());
+        glDisable(GL_BLEND);
+    }
+
     vbo->unbindArrays();
 }
 
-void BlurEffect::upscaleRenderToScreen(GLVertexBuffer *vbo, int vboStart, int blurRectCount, QMatrix4x4 screenProjection, QPoint windowPosition)
+void BlurEffect::upscaleRenderToScreen(GLVertexBuffer *vbo, int vboStart, int blurRectCount, const QMatrix4x4 &screenProjection, QPoint windowPosition)
 {
-    glActiveTexture(GL_TEXTURE0);
     m_renderTextures[1].bind();
 
-    if (m_noiseStrength > 0) {
-        m_shader->bind(BlurShader::NoiseSampleType);
-        m_shader->setTargetTextureSize(m_renderTextures[0].size() * GLRenderTarget::virtualScreenScale());
-        m_shader->setNoiseTextureSize(m_noiseTexture.size() * GLRenderTarget::virtualScreenScale());
-        m_shader->setTexturePosition(windowPosition * GLRenderTarget::virtualScreenScale());
-
-        glActiveTexture(GL_TEXTURE1);
-        m_noiseTexture.bind();
-    } else {
-        m_shader->bind(BlurShader::UpSampleType);
-        m_shader->setTargetTextureSize(m_renderTextures[0].size() * GLRenderTarget::virtualScreenScale());
-    }
+    m_shader->bind(BlurShader::UpSampleType);
+    m_shader->setTargetTextureSize(m_renderTextures[0].size() * GLRenderTarget::virtualScreenScale());
 
     m_shader->setOffset(m_offset);
     m_shader->setModelViewProjectionMatrix(screenProjection);
 
     //Render to the screen
     vbo->draw(GL_TRIANGLES, vboStart, blurRectCount);
+    m_shader->unbind();
+}
 
-    glActiveTexture(GL_TEXTURE0);
+void BlurEffect::applyNoise(GLVertexBuffer *vbo, int vboStart, int blurRectCount, const QMatrix4x4 &screenProjection, QPoint windowPosition)
+{
+    m_shader->bind(BlurShader::NoiseSampleType);
+    m_shader->setTargetTextureSize(m_renderTextures[0].size() * GLRenderTarget::virtualScreenScale());
+    m_shader->setNoiseTextureSize(m_noiseTexture->size() * GLRenderTarget::virtualScreenScale());
+    m_shader->setTexturePosition(windowPosition * GLRenderTarget::virtualScreenScale());
+
+    m_noiseTexture->bind();
+
+    m_shader->setOffset(m_offset);
+    m_shader->setModelViewProjectionMatrix(screenProjection);
+
+    vbo->draw(GL_TRIANGLES, vboStart, blurRectCount);
     m_shader->unbind();
 }
 
@@ -793,7 +818,7 @@ void BlurEffect::upSampleTexture(GLVertexBuffer *vbo, int blurRectCount)
     m_shader->unbind();
 }
 
-void BlurEffect::copyScreenSampleTexture(GLVertexBuffer *vbo, int blurRectCount, QRegion blurShape, QMatrix4x4 screenProjection)
+void BlurEffect::copyScreenSampleTexture(GLVertexBuffer *vbo, int blurRectCount, QRegion blurShape, const QMatrix4x4 &screenProjection)
 {
     m_shader->bind(BlurShader::CopySampleType);
 
