@@ -708,7 +708,7 @@ QByteArray GLPlatform::chipClassToString8(ChipClass chipClass)
 GLPlatform::GLPlatform()
     : m_driver(Driver_Unknown),
       m_chipClass(UnknownChipClass),
-      m_recommendedCompositor(XRenderCompositing),
+      m_recommendedCompositor(QPainterCompositing),
       m_glVersion(0),
       m_glslVersion(0),
       m_mesaVersion(0),
@@ -1030,13 +1030,13 @@ void GLPlatform::detect(OpenGLPlatformInterface platformInterface)
         }
 
         if (m_chipClass < R300) {
-            // fallback to XRender for R100 and R200
-            m_recommendedCompositor = XRenderCompositing;
+            // fallback to NoCompositing for R100 and R200
+            m_recommendedCompositor = NoCompositing;
         } else if (m_chipClass < R600) {
-            // XRender due to NPOT limitations not supported by KWin's shaders
-            m_recommendedCompositor = XRenderCompositing;
+            // NoCompositing due to NPOT limitations not supported by KWin's shaders
+            m_recommendedCompositor = NoCompositing;
         } else {
-            m_recommendedCompositor = OpenGL2Compositing;
+            m_recommendedCompositor = OpenGLCompositing;
         }
 
         if (driver() == Driver_R600G ||
@@ -1055,9 +1055,9 @@ void GLPlatform::detect(OpenGLPlatformInterface platformInterface)
         }
 
         if (m_chipClass < NV40) {
-            m_recommendedCompositor = XRenderCompositing;
+            m_recommendedCompositor = NoCompositing;
         } else {
-            m_recommendedCompositor = OpenGL2Compositing;
+            m_recommendedCompositor = OpenGLCompositing;
         }
 
         m_limitedNPOT = m_textureNPOT && m_chipClass < NV40;
@@ -1073,14 +1073,14 @@ void GLPlatform::detect(OpenGLPlatformInterface platformInterface)
         m_looseBinding = false;
 
         if (m_chipClass < I915) {
-            m_recommendedCompositor = XRenderCompositing;
+            m_recommendedCompositor = NoCompositing;
         } else {
-            m_recommendedCompositor = OpenGL2Compositing;
+            m_recommendedCompositor = OpenGLCompositing;
         }
     }
 
     if (isPanfrost()) {
-        m_recommendedCompositor = OpenGL2Compositing;
+        m_recommendedCompositor = OpenGLCompositing;
     }
 
     if (isMesaDriver() && platformInterface == EglPlatformInterface) {
@@ -1092,13 +1092,13 @@ void GLPlatform::detect(OpenGLPlatformInterface platformInterface)
 
     if (isSoftwareEmulation()) {
         if (m_driver < Driver_Llvmpipe) {
-            // we recommend XRender
-            m_recommendedCompositor = XRenderCompositing;
+            // we recommend QPainter
+            m_recommendedCompositor = QPainterCompositing;
             // Software emulation does not provide GLSL
             m_limitedGLSL = m_supportsGLSL = false;
         } else {
             // llvmpipe does support GLSL
-            m_recommendedCompositor = OpenGL2Compositing;
+            m_recommendedCompositor = OpenGLCompositing;
             m_limitedGLSL = false;
             m_supportsGLSL = true;
         }
@@ -1109,29 +1109,29 @@ void GLPlatform::detect(OpenGLPlatformInterface platformInterface)
             m_recommendedCompositor = NoCompositing;
         } else {
             // all other drivers support at least GLES 2
-            m_recommendedCompositor = OpenGL2Compositing;
+            m_recommendedCompositor = OpenGLCompositing;
         }
     }
 
     if (m_chipClass == UnknownChipClass && m_driver == Driver_Unknown) {
         // we don't know the hardware. Let's be optimistic and assume OpenGL compatible hardware
-        m_recommendedCompositor = OpenGL2Compositing;
+        m_recommendedCompositor = OpenGLCompositing;
         m_supportsGLSL = true;
     }
 
     if (isVirtualBox()) {
         m_virtualMachine = true;
-        m_recommendedCompositor = OpenGL2Compositing;
+        m_recommendedCompositor = OpenGLCompositing;
     }
 
     if (isVMware()) {
         m_virtualMachine = true;
-        m_recommendedCompositor = OpenGL2Compositing;
+        m_recommendedCompositor = OpenGLCompositing;
     }
 
     if (m_driver == Driver_Virgl) {
         m_virtualMachine = true;
-        m_recommendedCompositor = OpenGL2Compositing;
+        m_recommendedCompositor = OpenGLCompositing;
     }
 
     // and force back to shader supported on gles, we wouldn't have got a context if not supported
