@@ -167,6 +167,7 @@ void DrmGpu::initDrmResources()
 
 bool DrmGpu::updateOutputs()
 {
+    waitIdle();
     DrmScopedPointer<drmModeRes> resources(drmModeGetResources(m_fd));
     if (!resources) {
         qCWarning(KWIN_DRM) << "drmModeGetResources failed";
@@ -197,6 +198,7 @@ bool DrmGpu::updateOutputs()
             removeOutput(output);
         }
         m_connectors.removeOne(connector);
+        delete connector;
     }
 
     // find unused and connected connectors
@@ -222,10 +224,15 @@ bool DrmGpu::updateOutputs()
         plane->updateProperties();
     }
 
-    // delete current pipelines of active outputs
+    // stash away current pipelines of active outputs
+    QMap<DrmOutput*, DrmPipeline*> oldPipelines;
     for (const auto &output : qAsConst(m_drmOutputs)) {
+        if (!output->isEnabled()) {
+            // create render resources for findWorkingCombination
+            Q_EMIT outputEnabled(output);
+        }
         m_pipelines.removeOne(output->pipeline());
-        delete output->pipeline();
+        oldPipelines.insert(output, output->pipeline());
         output->setPipeline(nullptr);
     }
 
@@ -235,14 +242,33 @@ bool DrmGpu::updateOutputs()
             return c1->getProp(DrmConnector::PropertyIndex::CrtcId)->current() > c2->getProp(DrmConnector::PropertyIndex::CrtcId)->current();
         });
     }
-    const auto config = findWorkingCombination({}, connectedConnectors, m_crtcs, m_planes);
+    auto config = findWorkingCombination({}, connectedConnectors, m_crtcs, m_planes);
+    if (config.isEmpty() && !connectedConnectors.isEmpty()) {
+        qCCritical(KWIN_DRM) << "DrmGpu::findWorkingCombination failed to find any functional combinations! Reverting to the old configuration!";
+        for (auto it = oldPipelines.begin(); it != oldPipelines.end(); it++) {
+            it.value()->setOutput(it.key());
+            config << it.value();
+        }
+    } else {
+        for (const auto &pipeline : qAsConst(oldPipelines)) {
+            delete pipeline;
+        }
+    }
     m_pipelines << config;
 
-    for (const auto &pipeline : config) {
+    for (const auto &pipeline : qAsConst(config)) {
         auto output = pipeline->output();
         if (m_outputs.contains(output)) {
-            // try setting hardware rotation
-            output->updateTransform(output->transform());
+            // restore output properties
+            if (output->isEnabled()) {
+                output->updateTransform(output->transform());
+                if (output->dpmsMode() != AbstractWaylandOutput::DpmsMode::On) {
+                    pipeline->setActive(false);
+                }
+            } else {
+                pipeline->setActive(false);
+                Q_EMIT outputDisabled(output);
+            }
         } else {
             qCDebug(KWIN_DRM).nospace() << "New output on GPU " << m_devNode << ": " << pipeline->connector()->modelName();
             if (!output->initCursor(m_cursorSize)) {
@@ -332,7 +358,7 @@ bool DrmGpu::commitCombination(const QVector<DrmPipeline *> &pipelines)
         pipeline->setup();
     }
 
-    if (DrmPipeline::commitPipelines(pipelines, DrmPipeline::CommitMode::Commit)) {
+    if (DrmPipeline::commitPipelines(pipelines, DrmPipeline::CommitMode::Test)) {
         return true;
     } else {
         for (const auto &pipeline : qAsConst(pipelines)) {
@@ -426,7 +452,7 @@ static void pageFlipHandler(int fd, unsigned int frame, unsigned int sec, unsign
         return;
     }
 
-    // The static_cast<> here are for a 32-bit environment where 
+    // The static_cast<> here are for a 32-bit environment where
     // sizeof(time_t) == sizeof(unsigned int) == 4 . Putting @p sec
     // into a time_t cuts off the most-significant bit (after the
     // year 2038), similarly long can't hold all the bits of an
@@ -460,7 +486,9 @@ void DrmGpu::removeOutput(DrmOutput *output)
 {
     m_drmOutputs.removeOne(output);
     m_outputs.removeOne(output);
-    Q_EMIT outputDisabled(output);
+    if (output->isEnabled()) {
+        Q_EMIT outputDisabled(output);
+    }
     Q_EMIT outputRemoved(output);
     auto pipeline = output->m_pipeline;
     delete output;
@@ -500,7 +528,9 @@ DrmVirtualOutput *DrmGpu::createVirtualOutput()
 void DrmGpu::removeVirtualOutput(DrmVirtualOutput *output)
 {
     if (m_outputs.removeOne(output)) {
-        Q_EMIT outputDisabled(output);
+        if (output->isEnabled()) {
+            Q_EMIT outputDisabled(output);
+        }
         Q_EMIT outputRemoved(output);
         delete output;
     }
