@@ -114,7 +114,7 @@ bool DrmPipeline::present(const QSharedPointer<DrmBuffer> &buffer)
 
 bool DrmPipeline::atomicCommit()
 {
-    return commitPipelines({this}, CommitMode::CommitWithPageflipEvent);
+    return commitPipelines({this}, CommitMode::Commit);
 }
 
 bool DrmPipeline::commitPipelines(const QVector<DrmPipeline*> &pipelines, CommitMode mode)
@@ -152,14 +152,11 @@ bool DrmPipeline::commitPipelines(const QVector<DrmPipeline*> &pipelines, Commit
                 return failed();
             }
         }
-        if (mode != CommitMode::CommitWithPageflipEvent) {
-            flags &= ~DRM_MODE_PAGE_FLIP_EVENT;
-        }
-        if (drmModeAtomicCommit(pipelines[0]->m_gpu->fd(), req, (flags & (~DRM_MODE_PAGE_FLIP_EVENT)) | DRM_MODE_ATOMIC_TEST_ONLY, pipelines[0]->m_output) != 0) {
+        if (drmModeAtomicCommit(pipelines[0]->m_gpu->fd(), req, (flags & (~DRM_MODE_PAGE_FLIP_EVENT)) | DRM_MODE_ATOMIC_TEST_ONLY, pipelines[0]->output()) != 0) {
             qCWarning(KWIN_DRM) << "Atomic test for" << mode << "failed!" << strerror(errno);
             return failed();
         }
-        if (mode != CommitMode::Test && drmModeAtomicCommit(pipelines[0]->m_gpu->fd(), req, flags, pipelines[0]->m_output) != 0) {
+        if (mode != CommitMode::Test && drmModeAtomicCommit(pipelines[0]->m_gpu->fd(), req, flags, pipelines[0]->output()) != 0) {
             qCWarning(KWIN_DRM) << "Atomic commit failed! This should never happen!" << strerror(errno);
             return failed();
         }
@@ -269,6 +266,7 @@ bool DrmPipeline::modeset(int modeIndex)
         } else {
             m_crtc->setCurrent(m_primaryBuffer);
         }
+        m_connector->getProp(DrmConnector::PropertyIndex::Dpms)->setCurrent(DRM_MODE_DPMS_ON);
     }
     return true;
 }
@@ -281,32 +279,41 @@ bool DrmPipeline::checkTestBuffer()
     if (!m_active) {
         return true;
     }
-#if HAVE_GBM
     auto backend = m_gpu->eglBackend();
-    if (backend && m_output) {
-        auto buffer = backend->renderTestFrame(m_output);
-        if (buffer && buffer->bufferId()) {
-            m_oldTestBuffer = m_primaryBuffer;
-            m_primaryBuffer = buffer;
-            return true;
-        } else {
-            return false;
-        }
-    }
-    // we either don't have a DrmOutput or we're using QPainter
     QSharedPointer<DrmBuffer> buffer;
-    if (backend && m_gpu->gbmDevice()) {
+    // try to re-use buffers if possible.
+    const auto &checkBuffer = [this, backend, &buffer](const QSharedPointer<DrmBuffer> &buf){
+        const auto &mods = supportedModifiers(buf->format());
+        if (buf->format() == backend->drmFormat()
+            && (mods.isEmpty() || mods.contains(buf->modifier()))
+            && buf->size() == sourceSize()) {
+            buffer = buf;
+        }
+    };
+    if (m_primaryPlane && m_primaryPlane->next()) {
+        checkBuffer(m_primaryPlane->next());
+    } else if (m_primaryPlane && m_primaryPlane->current()) {
+        checkBuffer(m_primaryPlane->current());
+    } else if (m_crtc->next()) {
+        checkBuffer(m_crtc->next());
+    } else if (m_crtc->current()) {
+        checkBuffer(m_crtc->current());
+    }
+    // if we don't have a fitting buffer already, get or create one
+    if (buffer) {
+#if HAVE_GBM
+    } else if (backend && m_output) {
+        buffer = backend->renderTestFrame(m_output);
+    } else if (backend && m_gpu->gbmDevice()) {
         gbm_bo *bo = gbm_bo_create(m_gpu->gbmDevice(), sourceSize().width(), sourceSize().height(), GBM_FORMAT_XRGB8888, GBM_BO_USE_SCANOUT | GBM_BO_USE_RENDERING);
         if (!bo) {
             return false;
         }
         buffer = QSharedPointer<DrmGbmBuffer>::create(m_gpu, bo, nullptr);
+#endif
     } else {
         buffer = QSharedPointer<DrmDumbBuffer>::create(m_gpu, sourceSize());
     }
-#else
-    auto buffer = QSharedPointer<DrmDumbBuffer>::create(m_gpu, sourceSize());
-#endif
     if (buffer && buffer->bufferId()) {
         m_oldTestBuffer = m_primaryBuffer;
         m_primaryBuffer = buffer;
@@ -342,9 +349,9 @@ bool DrmPipeline::moveCursor(QPoint pos)
         return true;
     }
     if (drmModeMoveCursor(m_gpu->fd(), m_crtc->id(), pos.x(), pos.y()) != 0) {
-        m_cursor.pos = pos;
         return false;
     }
+    m_cursor.pos = pos;
     m_cursor.dirtyPos = false;
     return true;
 }
@@ -356,6 +363,8 @@ bool DrmPipeline::setActive(bool active)
         if (drmModeSetCursor(m_gpu->fd(), m_crtc->id(), 0, 0, 0) != 0) {
             qCWarning(KWIN_DRM) << "Could not set cursor:" << strerror(errno);
         }
+        m_cursor.dirtyBo = true;
+        m_cursor.dirtyPos = true;
     }
     bool success = false;
     auto mode = m_connector->currentMode().mode;
@@ -382,6 +391,11 @@ bool DrmPipeline::setActive(bool active)
             qCWarning(KWIN_DRM) << "Setting active failed: dpms property missing!";
         } else {
             success = drmModeConnectorSetProperty(m_gpu->fd(), m_connector->id(), dpmsProp->propId(), active ? DRM_MODE_DPMS_ON : DRM_MODE_DPMS_OFF) == 0;
+            if (success) {
+                dpmsProp->setPending(active ? DRM_MODE_DPMS_ON : DRM_MODE_DPMS_OFF);
+                dpmsProp->commitPending();
+                dpmsProp->commit();
+            }
         }
     }
     if (!success) {
@@ -391,6 +405,7 @@ bool DrmPipeline::setActive(bool active)
     if (m_active) {
         // enable cursor (again)
         setCursor(m_cursor.buffer, m_cursor.hotspot);
+        moveCursor(m_cursor.pos);
     }
     return success;
 }
@@ -461,7 +476,13 @@ bool DrmPipeline::setSyncMode(RenderLoopPrivate::SyncMode syncMode)
         vrrProp->setPending(vrr);
         return test();
     } else {
-        return drmModeObjectSetProperty(m_gpu->fd(), m_crtc->id(), DRM_MODE_OBJECT_CRTC, vrrProp->propId(), vrr) == 0;
+        bool ret = drmModeObjectSetProperty(m_gpu->fd(), m_crtc->id(), DRM_MODE_OBJECT_CRTC, vrrProp->propId(), vrr) == 0;
+        if (ret) {
+            vrrProp->setPending(vrr);
+            vrrProp->commitPending();
+            vrrProp->commit();
+        }
+        return ret;
     }
 }
 
@@ -623,7 +644,6 @@ void DrmPipeline::printDebugInfo() const
     }
     qCWarning(KWIN_DRM) << "Drm objects:";
     qCWarning(KWIN_DRM) << "connector" << m_connector->id();
-    auto list = m_connector->properties();
     printProps(m_connector);
     qCWarning(KWIN_DRM) << "crtc" << m_crtc->id();
     printProps(m_crtc);
