@@ -15,7 +15,7 @@
 namespace KWin
 {
 
-class EglMultiBackend : public QObject, public OpenGLBackend
+class EglMultiBackend : public OpenGLBackend
 {
     Q_OBJECT
 public:
@@ -24,19 +24,18 @@ public:
 
     void init() override;
 
-    QRegion beginFrame(int screenId) override;
-    void endFrame(int screenId, const QRegion &damage, const QRegion &damagedRegion) override;
-    bool scanout(int screenId, SurfaceItem *surfaceItem) override;
+    QRegion beginFrame(AbstractOutput *output) override;
+    void endFrame(AbstractOutput *output, const QRegion &damage, const QRegion &damagedRegion) override;
+    bool scanout(AbstractOutput *output, SurfaceItem *surfaceItem) override;
 
     bool makeCurrent() override;
     void doneCurrent() override;
 
-    SceneOpenGLTexturePrivate *createBackendTexture(SceneOpenGLTexture *texture) override;
+    PlatformSurfaceTexture *createPlatformSurfaceTextureInternal(SurfacePixmapInternal *pixmap) override;
+    PlatformSurfaceTexture *createPlatformSurfaceTextureWayland(SurfacePixmapWayland *pixmap) override;
     QSharedPointer<GLTexture> textureForOutput(AbstractOutput *requestedOutput) const override;
 
-    void screenGeometryChanged(const QSize &size) override;
-
-    bool directScanoutAllowed(int screen) const override;
+    bool directScanoutAllowed(AbstractOutput *output) const override;
 
 public Q_SLOTS:
     void addGpu(DrmGpu *gpu);
@@ -47,8 +46,7 @@ private:
     QVector<AbstractEglDrmBackend*> m_backends;
     bool m_initialized = false;
 
-    AbstractEglDrmBackend *findBackend(int screenId, int& internalScreenId) const;
-
+    AbstractEglDrmBackend *findBackend(AbstractOutput *output) const;
 };
 
 }
