@@ -16,8 +16,7 @@ namespace KWin
 {
 
 DrmQPainterBackend::DrmQPainterBackend(DrmBackend *backend, DrmGpu *gpu)
-    : QObject()
-    , QPainterBackend()
+    : QPainterBackend()
     , m_backend(backend)
     , m_gpu(gpu)
 {
@@ -27,7 +26,7 @@ DrmQPainterBackend::DrmQPainterBackend(DrmBackend *backend, DrmGpu *gpu)
     }
     connect(m_gpu, &DrmGpu::outputEnabled, this, &DrmQPainterBackend::initOutput);
     connect(m_gpu, &DrmGpu::outputDisabled, this,
-        [this] (DrmOutput *o) {
+        [this] (DrmAbstractOutput *o) {
             auto it = std::find_if(m_outputs.begin(), m_outputs.end(),
                 [o] (const Output &output) {
                     return output.output == o;
@@ -41,55 +40,50 @@ DrmQPainterBackend::DrmQPainterBackend(DrmBackend *backend, DrmGpu *gpu)
     );
 }
 
-void DrmQPainterBackend::initOutput(DrmOutput *output)
+void DrmQPainterBackend::initOutput(DrmAbstractOutput *output)
 {
     Output o;
     o.swapchain = QSharedPointer<DumbSwapchain>::create(m_gpu, output->pixelSize());
     o.output = output;
-    m_outputs << o;
-    connect(output, &DrmOutput::modeChanged, this,
+    m_outputs.insert(output, o);
+    connect(output, &DrmOutput::currentModeChanged, this,
         [output, this] {
-            auto it = std::find_if(m_outputs.begin(), m_outputs.end(),
-                [output] (const auto &o) {
-                    return o.output == output;
-                }
-            );
-            if (it == m_outputs.end()) {
-                return;
-            }
-            it->swapchain = QSharedPointer<DumbSwapchain>::create(m_gpu, output->pixelSize());
+            auto &o = m_outputs[output];
+            o.swapchain = QSharedPointer<DumbSwapchain>::create(m_gpu, output->pixelSize());
+            o.damageJournal.setCapacity(o.swapchain->slotCount());
         }
     );
 }
 
-QImage *DrmQPainterBackend::bufferForScreen(int screenId)
+QImage *DrmQPainterBackend::bufferForScreen(AbstractOutput *output)
 {
-    return m_outputs[screenId].swapchain->currentBuffer()->image();
+    return m_outputs[output].swapchain->currentBuffer()->image();
 }
 
-bool DrmQPainterBackend::needsFullRepaint(int screenId) const
+QRegion DrmQPainterBackend::beginFrame(AbstractOutput *output)
 {
-    Q_UNUSED(screenId)
-    return true;
+    Output *rendererOutput = &m_outputs[output];
+
+    int bufferAge;
+    rendererOutput->swapchain->acquireBuffer(&bufferAge);
+
+    return rendererOutput->damageJournal.accumulate(bufferAge, rendererOutput->output->geometry());
 }
 
-void DrmQPainterBackend::beginFrame(int screenId)
+void DrmQPainterBackend::endFrame(AbstractOutput *output, const QRegion &damage)
 {
-    m_outputs[screenId].swapchain->acquireBuffer();
-}
+    Output &rendererOutput = m_outputs[output];
+    DrmAbstractOutput *drmOutput = rendererOutput.output;
 
-void DrmQPainterBackend::endFrame(int screenId, int mask, const QRegion &damage)
-{
-    Q_UNUSED(mask)
-    Q_UNUSED(damage)
+    QSharedPointer<DrmDumbBuffer> back = rendererOutput.swapchain->currentBuffer();
+    rendererOutput.swapchain->releaseBuffer(back);
 
-    const Output &rendererOutput = m_outputs[screenId];
-    DrmOutput *drmOutput = rendererOutput.output;
-
-    if (!drmOutput->present(rendererOutput.swapchain->currentBuffer())) {
+    if (!drmOutput->present(back, drmOutput->geometry())) {
         RenderLoopPrivate *renderLoopPrivate = RenderLoopPrivate::get(drmOutput->renderLoop());
         renderLoopPrivate->notifyFrameFailed();
     }
+
+    rendererOutput.damageJournal.add(damage);
 }
 
 }
