@@ -30,6 +30,8 @@
 #include <sys/mman.h>
 #include <unistd.h>
 
+#include <libdrm/drm_fourcc.h>
+
 namespace KWin
 {
 
@@ -72,23 +74,34 @@ void PipeWireStream::newStreamParams()
 
     uint8_t paramsBuffer[1024];
     spa_pod_builder pod_builder = SPA_POD_BUILDER_INIT (paramsBuffer, sizeof (paramsBuffer));
+    int buffertypes;
 
     spa_rectangle resolution = SPA_RECTANGLE(uint32_t(m_resolution.width()), uint32_t(m_resolution.height()));
     const int cursorSize = Cursors::self()->currentCursor()->themeSize() * m_cursor.scale;
+    if (m_hasModifier) {
+        buffertypes = (1<<SPA_DATA_DmaBuf);
+    } else {
+        buffertypes = (1<<SPA_DATA_MemFd);
+    }
+
     const spa_pod *params[] = {
         (spa_pod*) spa_pod_builder_add_object(&pod_builder,
                                               SPA_TYPE_OBJECT_ParamBuffers, SPA_PARAM_Buffers,
                                               SPA_FORMAT_VIDEO_size, SPA_POD_Rectangle(&resolution),
                                               SPA_PARAM_BUFFERS_buffers, SPA_POD_CHOICE_RANGE_Int(16, 2, 16),
                                               SPA_PARAM_BUFFERS_blocks, SPA_POD_Int (1),
+                                              // TODO[no_use_linear]: stride, size and align should be dropped for dmabufs,
+                                              // or queried via test allocation
                                               SPA_PARAM_BUFFERS_stride, SPA_POD_Int(stride),
                                               SPA_PARAM_BUFFERS_size, SPA_POD_Int(stride * m_resolution.height()),
-                                              SPA_PARAM_BUFFERS_align, SPA_POD_Int(16)),
+                                              SPA_PARAM_BUFFERS_align, SPA_POD_Int(16),
+                                              SPA_PARAM_BUFFERS_dataType, SPA_POD_CHOICE_FLAGS_Int(buffertypes)),
         (spa_pod*) spa_pod_builder_add_object (&pod_builder,
                                                SPA_TYPE_OBJECT_ParamMeta, SPA_PARAM_Meta,
                                                SPA_PARAM_META_type, SPA_POD_Id (SPA_META_Cursor),
                                                SPA_PARAM_META_size, SPA_POD_Int (CURSOR_META_SIZE (cursorSize, cursorSize)))
     };
+
     pw_stream_update_params(pwStream, params, 2);
 }
 
@@ -100,6 +113,10 @@ void PipeWireStream::onStreamParamChanged(void *data, uint32_t id, const struct
 
     PipeWireStream *pw = static_cast<PipeWireStream *>(data);
     spa_format_video_raw_parse (format, &pw->videoFormat);
+    // TODO[explicit_modifiers]: check if modifier list or single modifier,
+    // make test allocation, fixate format, ...
+    // depends on how flexible kwin will become in that regard.
+    pw->m_hasModifier = spa_pod_find_prop(format, nullptr, SPA_FORMAT_VIDEO_modifier) != nullptr;
     qCDebug(KWIN_SCREENCAST) << "Stream format changed" << pw << pw->videoFormat.format;
     pw->newStreamParams();
 }
@@ -235,7 +252,7 @@ bool PipeWireStream::createStream()
     const QByteArray objname = "kwin-screencast-" + objectName().toUtf8();
     pwStream = pw_stream_new(pwCore->pwCore, objname, nullptr);
 
-    uint8_t buffer[1024];
+    uint8_t buffer[2048];
     spa_pod_builder podBuilder = SPA_POD_BUILDER_INIT(buffer, sizeof(buffer));
     spa_fraction minFramerate = SPA_FRACTION(1, 1);
     spa_fraction maxFramerate = SPA_FRACTION(25, 1);
@@ -243,25 +260,30 @@ bool PipeWireStream::createStream()
 
     spa_rectangle resolution = SPA_RECTANGLE(uint32_t(m_resolution.width()), uint32_t(m_resolution.height()));
 
+    // TODO[explicit_modifiers]: query modifiers supported/used by kwin
+    uint64_t modifier = DRM_FORMAT_MOD_INVALID;
+
+    const spa_pod *params[2];
+    int n_params;
+
     auto canCreateDmaBuf = [this] () -> bool {
         return QSharedPointer<DmaBufTexture>(kwinApp()->platform()->createDmaBufTexture(m_resolution));
     };
     const auto format = m_hasAlpha || canCreateDmaBuf() ? SPA_VIDEO_FORMAT_BGRA : SPA_VIDEO_FORMAT_BGR;
 
-    const spa_pod *param = (spa_pod*)spa_pod_builder_add_object(&podBuilder,
-                                        SPA_TYPE_OBJECT_Format, SPA_PARAM_EnumFormat,
-                                        SPA_FORMAT_mediaType, SPA_POD_Id(SPA_MEDIA_TYPE_video),
-                                        SPA_FORMAT_mediaSubtype, SPA_POD_Id(SPA_MEDIA_SUBTYPE_raw),
-                                        SPA_FORMAT_VIDEO_format, SPA_POD_CHOICE_ENUM_Id(format == SPA_VIDEO_FORMAT_BGRA ? 3 : 2,
-                                                                                        format, format, format == SPA_VIDEO_FORMAT_BGRA ? SPA_VIDEO_FORMAT_BGRx : SPA_VIDEO_FORMAT_UNKNOWN),
-                                        SPA_FORMAT_VIDEO_size, SPA_POD_Rectangle(&resolution),
-                                        SPA_FORMAT_VIDEO_framerate, SPA_POD_Fraction(&defaultFramerate),
-                                        SPA_FORMAT_VIDEO_maxFramerate, SPA_POD_CHOICE_RANGE_Fraction(&maxFramerate, &minFramerate, &maxFramerate));
+    if (canCreateDmaBuf()) {
+      params[0] = buildFormat(&podBuilder, format, &resolution, &defaultFramerate, &minFramerate, &maxFramerate, &modifier, 1);
+      params[1] = buildFormat(&podBuilder, format, &resolution, &defaultFramerate, &minFramerate, &maxFramerate, nullptr, 0);
+      n_params = 2;
+    } else {
+      params[0] = buildFormat(&podBuilder, format, &resolution, &defaultFramerate, &minFramerate, &maxFramerate, nullptr, 0);
+      n_params = 1;
+    }
 
     pw_stream_add_listener(pwStream, &streamListener, &pwStreamEvents, this);
     auto flags = pw_stream_flags(PW_STREAM_FLAG_DRIVER | PW_STREAM_FLAG_ALLOC_BUFFERS);
 
-    if (pw_stream_connect(pwStream, PW_DIRECTION_OUTPUT, SPA_ID_INVALID, flags, &param, 1) != 0) {
+    if (pw_stream_connect(pwStream, PW_DIRECTION_OUTPUT, SPA_ID_INVALID, flags, params, n_params) != 0) {
         qCWarning(KWIN_SCREENCAST) << "Could not connect to stream";
         pw_stream_destroy(pwStream);
         pwStream = nullptr;
@@ -478,6 +500,57 @@ void PipeWireStream::enqueue()
     m_pendingNotifier = nullptr;
 }
 
+spa_pod *PipeWireStream::buildFormat(struct spa_pod_builder *b, enum spa_video_format format, struct spa_rectangle *resolution,
+             struct spa_fraction *defaultFramerate, struct spa_fraction *minFramerate, struct spa_fraction *maxFramerate,
+             uint64_t *modifiers, int modifierCount)
+{
+    struct spa_pod_frame f[2];
+    int i, c;
+
+    spa_pod_builder_push_object(b, &f[0], SPA_TYPE_OBJECT_Format, SPA_PARAM_EnumFormat);
+    spa_pod_builder_add(b, SPA_FORMAT_mediaType, SPA_POD_Id(SPA_MEDIA_TYPE_video), 0);
+    spa_pod_builder_add(b, SPA_FORMAT_mediaSubtype, SPA_POD_Id(SPA_MEDIA_SUBTYPE_raw), 0);
+
+    /* format */
+    if (format == SPA_VIDEO_FORMAT_BGRA) {
+        /* announce equivalent format without alpha */
+        spa_pod_builder_add(b, SPA_FORMAT_VIDEO_format, SPA_POD_CHOICE_ENUM_Id(3, format, format, SPA_VIDEO_FORMAT_BGRx), 0);
+    } else {
+        spa_pod_builder_add(b, SPA_FORMAT_VIDEO_format, SPA_POD_Id(format), 0);
+    }
+    /* modifiers */
+    if (modifierCount > 0) {
+        // build an enumeration of modifiers
+        // TODO[explicit_modifiers]: Use SPA_POD_PROP_FLAG_DONT_FIXATE
+        // needs seperate fixateFormat method.
+        spa_pod_builder_prop(b, SPA_FORMAT_VIDEO_modifier, SPA_POD_PROP_FLAG_MANDATORY);
+        spa_pod_builder_push_choice(b, &f[1], SPA_CHOICE_Enum, 0);
+        // modifiers from the array
+        for (i = 0, c = 0; i < modifierCount; i++) {
+            spa_pod_builder_long(b, modifiers[i]);
+            if (c++ == 0) {
+                spa_pod_builder_long(b, modifiers[i]);
+            }
+        }
+        spa_pod_builder_pop(b, &f[1]);
+    }
+
+    /* frame size */
+    spa_pod_builder_add(b, SPA_FORMAT_VIDEO_size, SPA_POD_Rectangle(resolution), 0);
+
+    /* variable framerate */
+    spa_pod_builder_add(b, SPA_FORMAT_VIDEO_framerate, SPA_POD_Fraction(defaultFramerate), 0);
+
+    /* maximal framerate */
+    spa_pod_builder_add(b, SPA_FORMAT_VIDEO_maxFramerate,
+        SPA_POD_CHOICE_RANGE_Fraction(
+            SPA_POD_Fraction(maxFramerate),
+            SPA_POD_Fraction(minFramerate),
+            SPA_POD_Fraction(maxFramerate)),
+      0);
+    return (spa_pod*)spa_pod_builder_pop(b, &f[0]);
+}
+
 QRect PipeWireStream::cursorGeometry(Cursor *cursor) const
 {
     const auto position = (cursor->pos() - m_cursor.viewport.topLeft() - cursor->hotspot()) * m_cursor.scale;
